package view;

import model.*;
import javax.swing.*;
import javax.swing.undo.*;
import java.awt.event.*;

public class MenuView extends JMenuBar {
	private TriangleModel model;
	private JMenu file = new JMenu("File");
	private JMenu edit = new JMenu("Edit");
	
	private AbstractAction undoAction = new AbstractAction("Undo") {
		public void actionPerformed(ActionEvent e) {
			undoMgr.undo();
		}
	};
	
	private AbstractAction redoAction = new AbstractAction("Redo") {
		public void actionPerformed(ActionEvent e) {
			undoMgr.redo();
		}
	};
	
	AbstractAction quitAction = new AbstractAction("Quit") {
		public void actionPerformed(ActionEvent e) {
			System.exit(0);
		}
	};
	

	private static MenuView mvInstance = null;
	private static MyUndoManager undoMgr = null;
	
	public static MenuView getInstance(TriangleModel model) {
		if (mvInstance == null) {
			mvInstance = new MenuView(model);
		}
		return mvInstance;
	}

	private MenuView(TriangleModel model) {
		this.model = model;
		MenuView.undoMgr = new MyUndoManager();
		this.layoutView();
		this.registerListeners();
		
		this.model.addView(new IViewUpdate() {
			public void updateView() {
// Now handled by the custom UndoManager.  Might be other 
// menu items that need updating, though.
//				undoAction.setEnabled(undoMgr.canUndo());
//				redoAction.setEnabled(undoMgr.canRedo());
			}
		});
	}
	
	public static UndoManager getUndoManager() {
		return MenuView.undoMgr;
	}
	
	private void layoutView() {
		this.add(file);
		this.add(edit);
		
		this.quitAction.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.META_MASK));
		this.undoAction.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.META_MASK));
		this.redoAction.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.META_MASK | ActionEvent.SHIFT_MASK));
	}
	
	private void registerListeners() {
		this.file.add(quitAction);
		this.edit.add(undoAction);
		this.edit.add(redoAction);
		undoAction.setEnabled(undoMgr.canUndo());
		redoAction.setEnabled(undoMgr.canRedo());
	}
	
	// Can't figure out a better way to keep the menus updated...
	class MyUndoManager extends UndoManager {
		public void undo() {
			super.undo();
			undoAction.setEnabled(undoMgr.canUndo());
			redoAction.setEnabled(undoMgr.canRedo());
		}
		
		public void redo() {
			super.redo();
			undoAction.setEnabled(undoMgr.canUndo());
			redoAction.setEnabled(undoMgr.canRedo());
		}
		
		public boolean addEdit(UndoableEdit e) {
			boolean result = super.addEdit(e);
			undoAction.setEnabled(undoMgr.canUndo());
			redoAction.setEnabled(undoMgr.canRedo());	
			return result;
		}
	}
}
