package view;

import model.IViewUpdate;
import model.TriangleModel;

import javax.swing.*;
import java.awt.event.*;
import java.text.NumberFormat;
import javax.swing.undo.*;

/*
 * View a triangle as text:  numbers for the base, height, and hypotenuse.
 * 
 * @author Byron Weber Becker
 */
public class TextView extends JComponent {
	private TriangleModel model;
	private JTextField baseTF = new JTextField(10);
	private JTextField heightTF = new JTextField(10);
	private JTextField hypoTF = new JTextField(10);

	// To format numbers consistently in the text fields.
    private static final NumberFormat formatter = NumberFormat.getNumberInstance();

	public TextView(TriangleModel aModel) {
		super();
		this.model = aModel;
		this.layoutView();
		this.registerControllers();

		this.model.addView(new IViewUpdate() {
			public void updateView() {
				baseTF.setText(formatter.format(model.getBase()));
				heightTF.setText(formatter.format(model.getHeight()));
				hypoTF.setText(formatter.format(model.getHypotenuse()));
			}			
		});
	}

	private void layoutView() {
		this.setLayout(new FormLayout());
		this.add(new JLabel("Base:"));
		this.add(this.baseTF);
		this.add(new JLabel("Height:"));
		this.add(this.heightTF);
		this.add(new JLabel("Hypotenuse:"));
		this.add(this.hypoTF);

		// Don't allow the user to edit the hypotenuse
		this.hypoTF.setEditable(false);
		this.hypoTF.setEnabled(false);
	}

	private void registerControllers() {
		BaseController bc = new BaseController();
		HeightController hc = new HeightController();
		this.baseTF.addActionListener(bc);
		this.heightTF.addActionListener(hc);
		this.baseTF.addFocusListener(bc);
		this.heightTF.addFocusListener(hc);
	}

	private class BaseController implements ActionListener, FocusListener {
		
		public void actionPerformed(ActionEvent evt) {
			this.update();
		}
		
		public void focusGained(FocusEvent evt) {
			baseTF.selectAll();
		}

		public void focusLost(FocusEvent evt) {
			this.update();
		}
		
		private void update() {
			double base = Double.parseDouble(baseTF.getText());
			if (base != model.getBase()) {
				UndoableEdit cmd = model.getSetBaseCommand(base);
				MenuView.getUndoManager().addEdit(cmd);
				model.setBase(base);
			}
		}
	}

	private class HeightController implements ActionListener, FocusListener {
		
		public void actionPerformed(ActionEvent evt) {
			this.update();
		}
		
		public void focusGained(FocusEvent evt) {
			heightTF.selectAll();
		}

		public void focusLost(FocusEvent evt) {
			this.update();
		}
		
		private void update() {
			double height = Double.parseDouble(heightTF.getText());
			if (height != model.getHeight()) {
				UndoableEdit cmd = model.getSetHeightCommand(height);
				MenuView.getUndoManager().addEdit(cmd);
				model.setHeight(height);
			}
		}
	}
}